@@ -17,6 +17,7 @@ use std::{
     str::FromStr,
 };
 
+use zenoh_config::wrappers::EntityId;
 use zenoh_core::{Resolvable, Wait};
 use zenoh_keyexpr::{keyexpr, OwnedKeyExpr};
 use zenoh_protocol::{
@@ -35,7 +36,7 @@ pub(crate) enum KeyExprInner<'a> {
         expr_id: ExprId,
         mapping: Mapping,
         prefix_len: u32,
-        session_id: u16,
+        session_id: EntityId,
     },
     Owned(OwnedKeyExpr),
     Wire {
@@ -43,7 +44,7 @@ pub(crate) enum KeyExprInner<'a> {
         expr_id: ExprId,
         mapping: Mapping,
         prefix_len: u32,
-        session_id: u16,
+        session_id: EntityId,
     },
 }
 
@@ -104,7 +105,7 @@ impl<'a> KeyExpr<'a> {
         Self::try_from(t)
     }
 
-    /// Constructs key expression object to be used as dummy value
+    /// Constructs a key expression object to be used as a dummy value
     /// for empty objects. This method is not supposed to be called in user code,
     /// but may be used in language bindings (zenoh-c)
     #[zenoh_macros::internal]
@@ -173,7 +174,7 @@ impl<'a> KeyExpr<'a> {
         Self::new(t)
     }
 
-    /// Constructs an [`KeyExpr`] without checking [`keyexpr`]'s invariants
+    /// Constructs a [`KeyExpr`] without checking [`keyexpr`]'s invariants
     /// # Safety
     /// Key Expressions must follow some rules to be accepted by a Zenoh network.
     /// Messages addressed with invalid key expressions will be dropped.
@@ -294,32 +295,7 @@ impl<'a> KeyExpr<'a> {
         }
     }
 
-    /// Will return false and log a error in case of TryInto failure.
-    #[inline]
-    pub(crate) fn keyexpr_intersect<'b, L, R>(left: L, right: R) -> bool
-    where
-        L: TryInto<KeyExpr<'a>>,
-        R: TryInto<KeyExpr<'b>>,
-        L::Error: std::fmt::Display,
-        R::Error: std::fmt::Display,
-    {
-        match left.try_into() {
-            Ok(l) => match right.try_into() {
-                Ok(r) => {
-                    return l.intersects(&r);
-                }
-                Err(e) => {
-                    tracing::error!("{e}");
-                }
-            },
-            Err(e) => {
-                tracing::error!("{e}");
-            }
-        }
-        false
-    }
-
-    /// Will return false and log a error in case of TryInto failure.
+    /// Will return false and log an error in case of a `TryInto` failure.
     #[inline]
     pub(crate) fn keyexpr_include<'b, L, R>(left: L, right: R) -> bool
     where
@@ -633,7 +609,7 @@ impl<'a> UndeclarableSealed<&'a Session> for KeyExpr<'a> {
     }
 }
 
-/// A [`Resolvable`] returned when undeclaring a [`KeyExpr`].
+/// A [`Resolvable`] returned by [`Session::undeclare`] when undeclaring a [`KeyExpr`]
 ///
 /// # Examples
 /// ```
@@ -669,7 +645,7 @@ impl Wait for KeyExprUndeclaration<'_> {
                 if *session_id == session.0.id {
                     *expr_id
                 } else {
-                    return Err(zerror!("Failed to undeclare {}, as it was declared by an other Session", expr).into())
+                    return Err(zerror!("Failed to undeclare {}, as it was declared by another Session", expr).into())
                 }
             }
             KeyExprInner::BorrowedWire {
@@ -682,7 +658,7 @@ impl Wait for KeyExprUndeclaration<'_> {
                 if *session_id == session.0.id {
                     *expr_id
                 } else {
-                    return Err(zerror!("Failed to undeclare {}, as it was declared by an other Session", expr).into())
+                    return Err(zerror!("Failed to undeclare {}, as it was declared by another Session", expr).into())
                 }
             }
             _ => return Err(zerror!("Failed to undeclare {}, make sure you use the result of `Session::declare_keyexpr` to call `Session::undeclare`", expr).into()),
