@@ -18,7 +18,7 @@ use std::{
 };
 
 use zenoh_protocol::{
-    core::{key_expr::OwnedKeyExpr, WhatAmI},
+    core::WhatAmI,
     network::declare::{
         common::ext::WireExprType, ext, Declare, DeclareBody, DeclareSubscriber, SubscriberId,
         UndeclareSubscriber,
@@ -288,13 +288,13 @@ impl HatPubSubTrait for HatCode {
     }
 
     fn get_subscriptions(&self, tables: &Tables) -> Vec<(Arc<Resource>, Sources)> {
-        // Compute the list of known suscriptions (keys)
+        // Compute the list of known subscriptions (keys)
         let mut subs = HashMap::new();
         for src_face in tables.faces.values() {
             for sub in face_hat!(src_face).remote_subs.values() {
-                // Insert the key in the list of known suscriptions
+                // Insert the key in the list of known subscriptions
                 let srcs = subs.entry(sub.clone()).or_insert_with(Sources::empty);
-                // Append src_face as a suscription source in the proper list
+                // Append src_face as a subscription source in the proper list
                 match src_face.whatami {
                     WhatAmI::Router => srcs.routers.push(src_face.zid),
                     WhatAmI::Peer => srcs.peers.push(src_face.zid),
@@ -327,28 +327,46 @@ impl HatPubSubTrait for HatCode {
     fn compute_data_route(
         &self,
         tables: &Tables,
-        expr: &mut RoutingExpr,
+        expr: &RoutingExpr,
         source: NodeId,
         source_type: WhatAmI,
     ) -> Arc<Route> {
         let mut route = RouteBuilder::new();
-        let key_expr = expr.full_expr();
-        if key_expr.ends_with('/') {
+        let Some(key_expr) = expr.key_expr() else {
             return Arc::new(route.build());
-        }
+        };
         tracing::trace!(
             "compute_data_route({}, {:?}, {:?})",
             key_expr,
             source,
             source_type
         );
-        let key_expr = match OwnedKeyExpr::try_from(key_expr) {
-            Ok(ke) => ke,
-            Err(e) => {
-                tracing::warn!("Invalid KE reached the system: {}", e);
-                return Arc::new(route.build());
+
+        let matches = expr
+            .resource()
+            .as_ref()
+            .and_then(|res| res.context.as_ref())
+            .map(|ctx| Cow::from(&ctx.matches))
+            .unwrap_or_else(|| Cow::from(Resource::get_matches(tables, key_expr)));
+
+        for mres in matches.iter() {
+            let mres = mres.upgrade().unwrap();
+
+            for (sid, context) in &mres.session_ctxs {
+                if context.subs.is_some()
+                    && (source_type == WhatAmI::Client || context.face.whatami == WhatAmI::Client)
+                {
+                    route.insert(*sid, || {
+                        let wire_expr = expr.get_best_key(*sid);
+                        (
+                            context.face.clone(),
+                            wire_expr.to_owned(),
+                            NodeId::default(),
+                        )
+                    });
+                }
             }
-        };
+        }
 
         if source_type == WhatAmI::Client {
             for face in tables
@@ -356,46 +374,19 @@ impl HatPubSubTrait for HatCode {
                 .values()
                 .filter(|f| f.whatami != WhatAmI::Client)
             {
-                if !face.local_interests.values().any(|interest| {
-                    interest.finalized
-                        && interest.options.subscribers()
-                        && interest
-                            .res
-                            .as_ref()
-                            .map(|res| KeyExpr::keyexpr_include(res.expr(), expr.full_expr()))
-                            .unwrap_or(true)
-                }) || face_hat!(face)
-                    .remote_subs
-                    .values()
-                    .any(|sub| KeyExpr::keyexpr_intersect(sub.expr(), expr.full_expr()))
-                {
-                    let key_expr = Resource::get_best_key(expr.prefix, expr.suffix, face.id);
-                    route.insert(face.id, || {
-                        (face.clone(), key_expr.to_owned(), NodeId::default())
-                    });
-                }
+                route.try_insert(face.id, || {
+                    let has_interest_finalized = expr
+                        .resource()
+                        .and_then(|res| res.session_ctxs.get(&face.id))
+                        .is_some_and(|ctx| ctx.subscriber_interest_finalized);
+                    (!has_interest_finalized).then(|| {
+                        let wire_expr = expr.get_best_key(face.id);
+                        (face.clone(), wire_expr.to_owned(), NodeId::default())
+                    })
+                });
             }
         }
 
-        let res = Resource::get_resource(expr.prefix, expr.suffix);
-        let matches = res
-            .as_ref()
-            .and_then(|res| res.context.as_ref())
-            .map(|ctx| Cow::from(&ctx.matches))
-            .unwrap_or_else(|| Cow::from(Resource::get_matches(tables, &key_expr)));
-
-        for mres in matches.iter() {
-            let mres = mres.upgrade().unwrap();
-
-            for (sid, context) in &mres.session_ctxs {
-                if context.subs.is_some() && context.face.whatami == WhatAmI::Client {
-                    route.insert(*sid, || {
-                        let key_expr = Resource::get_best_key(expr.prefix, expr.suffix, *sid);
-                        (context.face.clone(), key_expr.to_owned(), NodeId::default())
-                    });
-                }
-            }
-        }
         Arc::new(route.build())
     }
 
@@ -410,28 +401,6 @@ impl HatPubSubTrait for HatCode {
         }
         tracing::trace!("get_matching_subscriptions({})", key_expr,);
 
-        for face in tables
-            .faces
-            .values()
-            .filter(|f| f.whatami != WhatAmI::Client)
-        {
-            if face.local_interests.values().any(|interest| {
-                interest.finalized
-                    && interest.options.subscribers()
-                    && interest
-                        .res
-                        .as_ref()
-                        .map(|res| KeyExpr::keyexpr_include(res.expr(), key_expr))
-                        .unwrap_or(true)
-            }) && face_hat!(face)
-                .remote_subs
-                .values()
-                .any(|sub| KeyExpr::keyexpr_intersect(sub.expr(), key_expr))
-            {
-                matching_subscriptions.insert(face.id, face.clone());
-            }
-        }
-
         let res = Resource::get_resource(&tables.root_res, key_expr);
         let matches = res
             .as_ref()
@@ -443,7 +412,7 @@ impl HatPubSubTrait for HatCode {
             let mres = mres.upgrade().unwrap();
 
             for (sid, context) in &mres.session_ctxs {
-                if context.subs.is_some() && context.face.whatami == WhatAmI::Client {
+                if context.subs.is_some() {
                     matching_subscriptions
                         .entry(*sid)
                         .or_insert_with(|| context.face.clone());
