@@ -19,10 +19,7 @@ use std::{
 
 use zenoh_protocol::{
     core::{
-        key_expr::{
-            include::{Includer, DEFAULT_INCLUDER},
-            OwnedKeyExpr,
-        },
+        key_expr::include::{Includer, DEFAULT_INCLUDER},
         WhatAmI,
     },
     network::declare::{
@@ -42,17 +39,12 @@ use crate::{
             tables::{QueryTargetQabl, QueryTargetQablSet, RoutingExpr, Tables},
         },
         hat::{HatQueriesTrait, SendDeclare, Sources},
+        router::{get_remote_qabl_info, merge_qabl_infos, update_queryable_info},
         RoutingContext,
     },
 };
 
 #[inline]
-fn merge_qabl_infos(mut this: QueryableInfoType, info: &QueryableInfoType) -> QueryableInfoType {
-    this.complete = this.complete || info.complete;
-    this.distance = std::cmp::min(this.distance, info.distance);
-    this
-}
-
 fn local_qabl_info(
     _tables: &Tables,
     res: &Arc<Resource>,
@@ -77,6 +69,37 @@ fn local_qabl_info(
         .unwrap_or(QueryableInfoType::DEFAULT)
 }
 
+#[inline]
+fn send_declare_queryable(
+    dst_face: &mut Arc<FaceState>,
+    res: &Arc<Resource>,
+    id: u32,
+    info: QueryableInfoType,
+    send_declare: &mut SendDeclare,
+) {
+    face_hat_mut!(dst_face)
+        .local_qabls
+        .insert(res.clone(), (id, info));
+    let key_expr = Resource::decl_key(res, dst_face, true);
+    send_declare(
+        &dst_face.primitives,
+        RoutingContext::with_expr(
+            Declare {
+                interest_id: None,
+                ext_qos: ext::QoSType::DECLARE,
+                ext_tstamp: None,
+                ext_nodeid: ext::NodeIdType::DEFAULT,
+                body: DeclareBody::DeclareQueryable(DeclareQueryable {
+                    id,
+                    wire_expr: key_expr,
+                    ext_info: info,
+                }),
+            },
+            res.expr().to_string(),
+        ),
+    );
+}
+
 fn propagate_simple_queryable(
     tables: &mut Tables,
     res: &Arc<Resource>,
@@ -85,44 +108,25 @@ fn propagate_simple_queryable(
 ) {
     let faces = tables.faces.values().cloned();
     for mut dst_face in faces {
-        let info = local_qabl_info(tables, res, &dst_face);
-        let current = face_hat!(dst_face).local_qabls.get(res);
         if src_face
             .as_ref()
-            .map(|src_face| dst_face.id != src_face.id)
+            .map(|src_face| {
+                dst_face.id != src_face.id
+                    && (src_face.whatami == WhatAmI::Client || dst_face.whatami == WhatAmI::Client)
+            })
             .unwrap_or(true)
-            && (current.is_none() || current.unwrap().1 != info)
-            && src_face
-                .as_ref()
-                .map(|src_face| {
-                    src_face.whatami == WhatAmI::Client || dst_face.whatami == WhatAmI::Client
-                })
-                .unwrap_or(true)
         {
-            let id = current
-                .map(|c| c.0)
-                .unwrap_or(face_hat!(dst_face).next_id.fetch_add(1, Ordering::SeqCst));
-            face_hat_mut!(&mut dst_face)
-                .local_qabls
-                .insert(res.clone(), (id, info));
-            let key_expr = Resource::decl_key(res, &mut dst_face, true);
-            send_declare(
-                &dst_face.primitives,
-                RoutingContext::with_expr(
-                    Declare {
-                        interest_id: None,
-                        ext_qos: ext::QoSType::DECLARE,
-                        ext_tstamp: None,
-                        ext_nodeid: ext::NodeIdType::DEFAULT,
-                        body: DeclareBody::DeclareQueryable(DeclareQueryable {
-                            id,
-                            wire_expr: key_expr,
-                            ext_info: info,
-                        }),
-                    },
-                    res.expr().to_string(),
-                ),
-            );
+            if let Some(&(current_id, current_info)) = face_hat!(dst_face).local_qabls.get(res) {
+                let info = local_qabl_info(tables, res, &dst_face);
+                if current_info != info {
+                    let id = current_id;
+                    send_declare_queryable(&mut dst_face, res, id, info, send_declare);
+                }
+            } else {
+                let info = local_qabl_info(tables, res, &dst_face);
+                let id = face_hat!(dst_face).next_id.fetch_add(1, Ordering::SeqCst);
+                send_declare_queryable(&mut dst_face, res, id, info, send_declare);
+            }
         }
     }
 }
@@ -144,7 +148,9 @@ fn register_simple_queryable(
         )
         .qabl = Some(*qabl_info);
     }
-    face_hat_mut!(face).remote_qabls.insert(id, res.clone());
+    face_hat_mut!(face)
+        .remote_qabls
+        .insert(id, (res.clone(), *qabl_info));
 }
 
 fn declare_simple_queryable(
@@ -206,15 +212,8 @@ pub(super) fn undeclare_simple_queryable(
     res: &mut Arc<Resource>,
     send_declare: &mut SendDeclare,
 ) {
-    if !face_hat_mut!(face)
-        .remote_qabls
-        .values()
-        .any(|s| *s == *res)
-    {
-        if let Some(ctx) = get_mut_unchecked(res).session_ctxs.get_mut(&face.id) {
-            get_mut_unchecked(ctx).qabl = None;
-        }
-
+    let remote_qabl_info = get_remote_qabl_info(&face_hat_mut!(face).remote_qabls, res);
+    if update_queryable_info(res, face.id, &remote_qabl_info) {
         let mut simple_qabls = simple_qabls(res);
         if simple_qabls.is_empty() {
             propagate_forget_simple_queryable(tables, res, send_declare);
@@ -251,7 +250,7 @@ fn forget_simple_queryable(
     id: QueryableId,
     send_declare: &mut SendDeclare,
 ) -> Option<Arc<Resource>> {
-    if let Some(mut res) = face_hat_mut!(face).remote_qabls.remove(&id) {
+    if let Some((mut res, _)) = face_hat_mut!(face).remote_qabls.remove(&id) {
         undeclare_simple_queryable(tables, face, &mut res, send_declare);
         Some(res)
     } else {
@@ -270,7 +269,7 @@ pub(super) fn queries_new_face(
         .cloned()
         .collect::<Vec<Arc<FaceState>>>()
     {
-        for qabl in face_hat!(face).remote_qabls.values() {
+        for (qabl, _) in face_hat!(face).remote_qabls.values() {
             propagate_simple_queryable(tables, qabl, Some(&mut face.clone()), send_declare);
         }
     }
@@ -310,7 +309,7 @@ impl HatQueriesTrait for HatCode {
         // Compute the list of known queryables (keys)
         let mut qabls = HashMap::new();
         for src_face in tables.faces.values() {
-            for qabl in face_hat!(src_face).remote_qabls.values() {
+            for (ref qabl, _) in face_hat!(src_face).remote_qabls.values() {
                 // Insert the key in the list of known queryables
                 let srcs = qabls.entry(qabl.clone()).or_insert_with(Sources::empty);
                 // Append src_face as a queryable source in the proper list
@@ -346,63 +345,27 @@ impl HatQueriesTrait for HatCode {
     fn compute_query_route(
         &self,
         tables: &Tables,
-        expr: &mut RoutingExpr,
+        expr: &RoutingExpr,
         source: NodeId,
         source_type: WhatAmI,
     ) -> Arc<QueryTargetQablSet> {
         let mut route = QueryTargetQablSet::new();
-        let key_expr = expr.full_expr();
-        if key_expr.ends_with('/') {
+        let Some(key_expr) = expr.key_expr() else {
             return EMPTY_ROUTE.clone();
-        }
+        };
         tracing::trace!(
             "compute_query_route({}, {:?}, {:?})",
             key_expr,
             source,
             source_type
         );
-        let key_expr = match OwnedKeyExpr::try_from(key_expr) {
-            Ok(ke) => ke,
-            Err(e) => {
-                tracing::warn!("Invalid KE reached the system: {}", e);
-                return EMPTY_ROUTE.clone();
-            }
-        };
 
-        if source_type == WhatAmI::Client {
-            for face in tables
-                .faces
-                .values()
-                .filter(|f| f.whatami != WhatAmI::Client)
-            {
-                if !face.local_interests.values().any(|interest| {
-                    interest.finalized
-                        && interest.options.queryables()
-                        && interest
-                            .res
-                            .as_ref()
-                            .map(|res| KeyExpr::keyexpr_include(res.expr(), expr.full_expr()))
-                            .unwrap_or(true)
-                }) || face_hat!(face)
-                    .remote_qabls
-                    .values()
-                    .any(|qbl| KeyExpr::keyexpr_intersect(qbl.expr(), expr.full_expr()))
-                {
-                    let key_expr = Resource::get_best_key(expr.prefix, expr.suffix, face.id);
-                    route.push(QueryTargetQabl {
-                        direction: (face.clone(), key_expr.to_owned(), NodeId::default()),
-                        info: None,
-                    });
-                }
-            }
-        }
-
-        let res = Resource::get_resource(expr.prefix, expr.suffix);
-        let matches = res
+        let matches = expr
+            .resource()
             .as_ref()
             .and_then(|res| res.context.as_ref())
             .map(|ctx| Cow::from(&ctx.matches))
-            .unwrap_or_else(|| Cow::from(Resource::get_matches(tables, &key_expr)));
+            .unwrap_or_else(|| Cow::from(Resource::get_matches(tables, key_expr)));
 
         for mres in matches.iter() {
             let mres = mres.upgrade().unwrap();
@@ -413,6 +376,27 @@ impl HatQueriesTrait for HatCode {
                 }
             }
         }
+
+        if source_type == WhatAmI::Client {
+            for face in tables
+                .faces
+                .values()
+                .filter(|f| f.whatami != WhatAmI::Client)
+            {
+                let has_interest_finalized = expr
+                    .resource()
+                    .and_then(|res| res.session_ctxs.get(&face.id))
+                    .is_some_and(|ctx| ctx.queryable_interest_finalized);
+                if !has_interest_finalized {
+                    let wire_expr = expr.get_best_key(face.id);
+                    route.push(QueryTargetQabl {
+                        direction: (face.clone(), wire_expr.to_owned(), NodeId::default()),
+                        info: None,
+                    });
+                }
+            }
+        }
+
         route.sort_by_key(|qabl| qabl.info.map_or(u16::MAX, |i| i.distance));
         Arc::new(route)
     }
@@ -432,36 +416,6 @@ impl HatQueriesTrait for HatCode {
             key_expr,
             complete
         );
-        for face in tables
-            .faces
-            .values()
-            .filter(|f| f.whatami != WhatAmI::Client)
-        {
-            if face.local_interests.values().any(|interest| {
-                interest.finalized
-                    && interest.options.queryables()
-                    && interest
-                        .res
-                        .as_ref()
-                        .map(|res| KeyExpr::keyexpr_include(res.expr(), key_expr))
-                        .unwrap_or(true)
-            }) && face_hat!(face)
-                .remote_qabls
-                .values()
-                .any(|qbl| match complete {
-                    true => {
-                        qbl.session_ctxs
-                            .get(&face.id)
-                            .and_then(|sc| sc.qabl)
-                            .is_some_and(|q| q.complete)
-                            && KeyExpr::keyexpr_include(qbl.expr(), key_expr)
-                    }
-                    false => KeyExpr::keyexpr_intersect(qbl.expr(), key_expr),
-                })
-            {
-                matching_queryables.insert(face.id, face.clone());
-            }
-        }
 
         let res = Resource::get_resource(&tables.root_res, key_expr);
         let matches = res
@@ -476,12 +430,10 @@ impl HatQueriesTrait for HatCode {
                 continue;
             }
             for (sid, context) in &mres.session_ctxs {
-                if context.face.whatami == WhatAmI::Client
-                    && match complete {
-                        true => context.qabl.is_some_and(|q| q.complete),
-                        false => context.qabl.is_some(),
-                    }
-                {
+                if match complete {
+                    true => context.qabl.is_some_and(|q| q.complete),
+                    false => context.qabl.is_some(),
+                } {
                     matching_queryables
                         .entry(*sid)
                         .or_insert_with(|| context.face.clone());
