@@ -20,10 +20,7 @@ use std::{
 use petgraph::graph::NodeIndex;
 use zenoh_protocol::{
     core::{
-        key_expr::{
-            include::{Includer, DEFAULT_INCLUDER},
-            OwnedKeyExpr,
-        },
+        key_expr::include::{Includer, DEFAULT_INCLUDER},
         WhatAmI, ZenohIdProto,
     },
     network::{
@@ -51,19 +48,16 @@ use crate::{
                 tables::{QueryTargetQabl, QueryTargetQablSet, RoutingExpr, Tables},
             },
             hat::{CurrentFutureTrait, HatQueriesTrait, SendDeclare, Sources},
-            router::disable_matches_query_routes,
+            router::{
+                disable_matches_query_routes, get_remote_qabl_info, merge_qabl_infos,
+                update_queryable_info,
+            },
             RoutingContext,
         },
     },
 };
 
 #[inline]
-fn merge_qabl_infos(mut this: QueryableInfoType, info: &QueryableInfoType) -> QueryableInfoType {
-    this.complete = this.complete || info.complete;
-    this.distance = std::cmp::min(this.distance, info.distance);
-    this
-}
-
 fn local_peer_qabl_info(_tables: &Tables, res: &Arc<Resource>) -> QueryableInfoType {
     res.session_ctxs
         .values()
@@ -80,6 +74,7 @@ fn local_peer_qabl_info(_tables: &Tables, res: &Arc<Resource>) -> QueryableInfoT
         .unwrap_or(QueryableInfoType::DEFAULT)
 }
 
+#[inline]
 fn local_qabl_info(
     tables: &Tables,
     res: &Arc<Resource>,
@@ -169,6 +164,37 @@ fn send_sourced_queryable_to_net_children(
     }
 }
 
+#[inline]
+fn send_declare_queryable(
+    dst_face: &mut Arc<FaceState>,
+    res: &Arc<Resource>,
+    id: u32,
+    info: QueryableInfoType,
+    send_declare: &mut SendDeclare,
+) {
+    face_hat_mut!(dst_face)
+        .local_qabls
+        .insert(res.clone(), (id, info));
+    let key_expr = Resource::decl_key(res, dst_face, super::push_declaration_profile(dst_face));
+    send_declare(
+        &dst_face.primitives,
+        RoutingContext::with_expr(
+            Declare {
+                interest_id: None,
+                ext_qos: ext::QoSType::DECLARE,
+                ext_tstamp: None,
+                ext_nodeid: ext::NodeIdType::DEFAULT,
+                body: DeclareBody::DeclareQueryable(DeclareQueryable {
+                    id,
+                    wire_expr: key_expr,
+                    ext_info: info,
+                }),
+            },
+            res.expr().to_string(),
+        ),
+    );
+}
+
 fn propagate_simple_queryable(
     tables: &mut Tables,
     res: &Arc<Resource>,
@@ -177,44 +203,32 @@ fn propagate_simple_queryable(
 ) {
     let faces = tables.faces.values().cloned();
     for mut dst_face in faces {
-        let info = local_qabl_info(tables, res, &dst_face);
-        let current = face_hat!(dst_face).local_qabls.get(res);
         if src_face
             .as_ref()
             .map(|src_face| dst_face.id != src_face.id)
             .unwrap_or(true)
-            && (current.is_none() || current.unwrap().1 != info)
             && dst_face.whatami == WhatAmI::Client
-            && face_hat!(dst_face)
+        {
+            if let Some(&(current_id, current_info)) = face_hat!(dst_face).local_qabls.get(res) {
+                let info = local_qabl_info(tables, res, &dst_face);
+                if current_info != info
+                    && face_hat!(dst_face)
+                        .remote_interests
+                        .values()
+                        .any(|i| i.options.queryables() && i.matches(res))
+                {
+                    let id = current_id;
+                    send_declare_queryable(&mut dst_face, res, id, info, send_declare);
+                }
+            } else if face_hat!(dst_face)
                 .remote_interests
                 .values()
                 .any(|i| i.options.queryables() && i.matches(res))
-        {
-            let id = current
-                .map(|c| c.0)
-                .unwrap_or(face_hat!(dst_face).next_id.fetch_add(1, Ordering::SeqCst));
-            face_hat_mut!(&mut dst_face)
-                .local_qabls
-                .insert(res.clone(), (id, info));
-            let push_declaration = push_declaration_profile(&dst_face);
-            let key_expr = Resource::decl_key(res, &mut dst_face, push_declaration);
-            send_declare(
-                &dst_face.primitives,
-                RoutingContext::with_expr(
-                    Declare {
-                        interest_id: None,
-                        ext_qos: ext::QoSType::DECLARE,
-                        ext_tstamp: None,
-                        ext_nodeid: ext::NodeIdType::DEFAULT,
-                        body: DeclareBody::DeclareQueryable(DeclareQueryable {
-                            id,
-                            wire_expr: key_expr,
-                            ext_info: info,
-                        }),
-                    },
-                    res.expr().to_string(),
-                ),
-            );
+            {
+                let info = local_qabl_info(tables, res, &dst_face);
+                let id = face_hat!(dst_face).next_id.fetch_add(1, Ordering::SeqCst);
+                send_declare_queryable(&mut dst_face, res, id, info, send_declare);
+            }
         }
     }
 }
@@ -311,7 +325,9 @@ fn register_simple_queryable(
         )
         .qabl = Some(*qabl_info);
     }
-    face_hat_mut!(face).remote_qabls.insert(id, res.clone());
+    face_hat_mut!(face)
+        .remote_qabls
+        .insert(id, (res.clone(), *qabl_info));
 }
 
 fn declare_simple_queryable(
@@ -542,15 +558,9 @@ pub(super) fn undeclare_simple_queryable(
     res: &mut Arc<Resource>,
     send_declare: &mut SendDeclare,
 ) {
-    if !face_hat_mut!(face)
-        .remote_qabls
-        .values()
-        .any(|s| *s == *res)
-    {
-        if let Some(ctx) = get_mut_unchecked(res).session_ctxs.get_mut(&face.id) {
-            get_mut_unchecked(ctx).qabl = None;
-        }
+    let remote_qabl_info = get_remote_qabl_info(&face_hat_mut!(face).remote_qabls, res);
 
+    if update_queryable_info(res, face.id, &remote_qabl_info) {
         let mut simple_qabls = simple_qabls(res);
         let linkstatepeer_qabls = remote_linkstatepeer_qabls(tables, res);
 
@@ -631,7 +641,7 @@ fn forget_simple_queryable(
     id: QueryableId,
     send_declare: &mut SendDeclare,
 ) -> Option<Arc<Resource>> {
-    if let Some(mut res) = face_hat_mut!(face).remote_qabls.remove(&id) {
+    if let Some((mut res, _)) = face_hat_mut!(face).remote_qabls.remove(&id) {
         undeclare_simple_queryable(tables, face, &mut res, send_declare);
         Some(res)
     } else {
@@ -699,7 +709,7 @@ pub(super) fn queries_tree_change(tables: &mut Tables, new_children: &[Vec<NodeI
 #[inline]
 fn insert_target_for_qabls(
     route: &mut QueryTargetQablSet,
-    expr: &mut RoutingExpr,
+    expr: &RoutingExpr,
     tables: &Tables,
     net: &Network,
     source: NodeId,
@@ -715,10 +725,9 @@ fn insert_target_for_qabls(
                         if net.graph.contains_node(direction) {
                             if let Some(face) = tables.get_face(&net.graph[direction].zid) {
                                 if net.distances.len() > qabl_idx.index() {
-                                    let key_expr =
-                                        Resource::get_best_key(expr.prefix, expr.suffix, face.id);
+                                    let wire_expr = expr.get_best_key(face.id);
                                     route.push(QueryTargetQabl {
-                                        direction: (face.clone(), key_expr.to_owned(), source),
+                                        direction: (face.clone(), wire_expr.to_owned(), source),
                                         info: Some(QueryableInfoType {
                                             complete: complete && qabl_info.complete,
                                             distance: net.distances[qabl_idx.index()] as u16,
@@ -980,34 +989,26 @@ impl HatQueriesTrait for HatCode {
     fn compute_query_route(
         &self,
         tables: &Tables,
-        expr: &mut RoutingExpr,
+        expr: &RoutingExpr,
         source: NodeId,
         source_type: WhatAmI,
     ) -> Arc<QueryTargetQablSet> {
         let mut route = QueryTargetQablSet::new();
-        let key_expr = expr.full_expr();
-        if key_expr.ends_with('/') {
+        let Some(key_expr) = expr.key_expr() else {
             return EMPTY_ROUTE.clone();
-        }
+        };
         tracing::trace!(
             "compute_query_route({}, {:?}, {:?})",
             key_expr,
             source,
             source_type
         );
-        let key_expr = match OwnedKeyExpr::try_from(key_expr) {
-            Ok(ke) => ke,
-            Err(e) => {
-                tracing::warn!("Invalid KE reached the system: {}", e);
-                return EMPTY_ROUTE.clone();
-            }
-        };
-        let res = Resource::get_resource(expr.prefix, expr.suffix);
-        let matches = res
+        let matches = expr
+            .resource()
             .as_ref()
             .and_then(|res| res.context.as_ref())
             .map(|ctx| Cow::from(&ctx.matches))
-            .unwrap_or_else(|| Cow::from(Resource::get_matches(tables, &key_expr)));
+            .unwrap_or_else(|| Cow::from(Resource::get_matches(tables, key_expr)));
 
         for mres in matches.iter() {
             let mres = mres.upgrade().unwrap();
