@@ -188,13 +188,10 @@ impl HatBaseTrait for HatCode {
             }
         }
         if face.state.whatami == WhatAmI::Peer {
+            let face_id = face.state.id;
             get_mut_unchecked(&mut face.state).local_interests.insert(
                 INITIAL_INTEREST_ID,
-                InterestState {
-                    options: InterestOptions::ALL,
-                    res: None,
-                    finalized: false,
-                },
+                InterestState::new(face_id, InterestOptions::ALL, None, false),
             );
         }
 
@@ -276,7 +273,7 @@ impl HatBaseTrait for HatCode {
         }
 
         let mut qabls_matches = vec![];
-        for (_id, mut res) in hat_face.remote_qabls.drain() {
+        for (_id, (mut res, _)) in hat_face.remote_qabls.drain() {
             get_mut_unchecked(&mut res).session_ctxs.remove(&face.id);
             undeclare_simple_queryable(&mut wtables, &mut face_clone, &mut res, send_declare);
 
@@ -367,7 +364,7 @@ impl HatBaseTrait for HatCode {
     }
 
     #[inline]
-    fn ingress_filter(&self, _tables: &Tables, _face: &FaceState, _expr: &mut RoutingExpr) -> bool {
+    fn ingress_filter(&self, _tables: &Tables, _face: &FaceState, _expr: &RoutingExpr) -> bool {
         true
     }
 
@@ -377,7 +374,7 @@ impl HatBaseTrait for HatCode {
         _tables: &Tables,
         src_face: &FaceState,
         out_face: &Arc<FaceState>,
-        _expr: &mut RoutingExpr,
+        _expr: &RoutingExpr,
     ) -> bool {
         src_face.id != out_face.id
             && (out_face.mcast_group.is_none()
@@ -405,7 +402,7 @@ struct HatFace {
     local_tokens: HashMap<Arc<Resource>, TokenId>,
     remote_tokens: HashMap<TokenId, Arc<Resource>>,
     local_qabls: HashMap<Arc<Resource>, (QueryableId, QueryableInfoType)>,
-    remote_qabls: HashMap<QueryableId, Arc<Resource>>,
+    remote_qabls: HashMap<QueryableId, (Arc<Resource>, QueryableInfoType)>,
 }
 
 impl HatFace {
