@@ -19,10 +19,7 @@ use std::{
 
 use zenoh_protocol::{
     core::{
-        key_expr::{
-            include::{Includer, DEFAULT_INCLUDER},
-            OwnedKeyExpr,
-        },
+        key_expr::include::{Includer, DEFAULT_INCLUDER},
         WhatAmI,
     },
     network::{
@@ -47,17 +44,12 @@ use crate::{
         hat::{
             p2p_peer::initial_interest, CurrentFutureTrait, HatQueriesTrait, SendDeclare, Sources,
         },
+        router::{get_remote_qabl_info, merge_qabl_infos, update_queryable_info},
         RoutingContext,
     },
 };
 
 #[inline]
-fn merge_qabl_infos(mut this: QueryableInfoType, info: &QueryableInfoType) -> QueryableInfoType {
-    this.complete = this.complete || info.complete;
-    this.distance = std::cmp::min(this.distance, info.distance);
-    this
-}
-
 fn local_qabl_info(
     _tables: &Tables,
     res: &Arc<Resource>,
@@ -82,6 +74,37 @@ fn local_qabl_info(
         .unwrap_or(QueryableInfoType::DEFAULT)
 }
 
+#[inline]
+fn send_declare_queryable(
+    dst_face: &mut Arc<FaceState>,
+    res: &Arc<Resource>,
+    id: u32,
+    info: QueryableInfoType,
+    send_declare: &mut SendDeclare,
+) {
+    face_hat_mut!(dst_face)
+        .local_qabls
+        .insert(res.clone(), (id, info));
+    let key_expr = Resource::decl_key(res, dst_face, super::push_declaration_profile(dst_face));
+    send_declare(
+        &dst_face.primitives,
+        RoutingContext::with_expr(
+            Declare {
+                interest_id: None,
+                ext_qos: ext::QoSType::DECLARE,
+                ext_tstamp: None,
+                ext_nodeid: ext::NodeIdType::DEFAULT,
+                body: DeclareBody::DeclareQueryable(DeclareQueryable {
+                    id,
+                    wire_expr: key_expr,
+                    ext_info: info,
+                }),
+            },
+            res.expr().to_string(),
+        ),
+    );
+}
+
 #[inline]
 fn propagate_simple_queryable_to(
     tables: &mut Tables,
@@ -90,49 +113,36 @@ fn propagate_simple_queryable_to(
     src_face: &Option<&mut Arc<FaceState>>,
     send_declare: &mut SendDeclare,
 ) {
-    let info = local_qabl_info(tables, res, dst_face);
-    let current = face_hat!(dst_face).local_qabls.get(res);
     if src_face
         .as_ref()
-        .map(|src_face| dst_face.id != src_face.id)
+        .map(|src_face| {
+            dst_face.id != src_face.id
+                && (src_face.whatami == WhatAmI::Client || dst_face.whatami == WhatAmI::Client)
+        })
         .unwrap_or(true)
-        && (current.is_none() || current.unwrap().1 != info)
-        && (dst_face.whatami != WhatAmI::Client
+    {
+        if let Some(&(current_id, current_info)) = face_hat!(dst_face).local_qabls.get(res) {
+            let info = local_qabl_info(tables, res, dst_face);
+            if current_info != info
+                && (dst_face.whatami != WhatAmI::Client
+                    || face_hat!(dst_face)
+                        .remote_interests
+                        .values()
+                        .any(|i| i.options.queryables() && i.matches(res)))
+            {
+                let id = current_id;
+                send_declare_queryable(dst_face, res, id, info, send_declare);
+            }
+        } else if dst_face.whatami != WhatAmI::Client
             || face_hat!(dst_face)
                 .remote_interests
                 .values()
-                .any(|i| i.options.queryables() && i.matches(res)))
-        && src_face
-            .as_ref()
-            .map(|src_face| {
-                src_face.whatami == WhatAmI::Client || dst_face.whatami == WhatAmI::Client
-            })
-            .unwrap_or(true)
-    {
-        let id = current
-            .map(|c| c.0)
-            .unwrap_or(face_hat!(dst_face).next_id.fetch_add(1, Ordering::SeqCst));
-        face_hat_mut!(dst_face)
-            .local_qabls
-            .insert(res.clone(), (id, info));
-        let key_expr = Resource::decl_key(res, dst_face, super::push_declaration_profile(dst_face));
-        send_declare(
-            &dst_face.primitives,
-            RoutingContext::with_expr(
-                Declare {
-                    interest_id: None,
-                    ext_qos: ext::QoSType::DECLARE,
-                    ext_tstamp: None,
-                    ext_nodeid: ext::NodeIdType::DEFAULT,
-                    body: DeclareBody::DeclareQueryable(DeclareQueryable {
-                        id,
-                        wire_expr: key_expr,
-                        ext_info: info,
-                    }),
-                },
-                res.expr().to_string(),
-            ),
-        );
+                .any(|i| i.options.queryables() && i.matches(res))
+        {
+            let info = local_qabl_info(tables, res, dst_face);
+            let id = face_hat!(dst_face).next_id.fetch_add(1, Ordering::SeqCst);
+            send_declare_queryable(dst_face, res, id, info, send_declare);
+        }
     }
 }
 
@@ -169,7 +179,9 @@ fn register_simple_queryable(
         )
         .qabl = Some(*qabl_info);
     }
-    face_hat_mut!(face).remote_qabls.insert(id, res.clone());
+    face_hat_mut!(face)
+        .remote_qabls
+        .insert(id, (res.clone(), *qabl_info));
 }
 
 fn declare_simple_queryable(
@@ -268,15 +280,9 @@ pub(super) fn undeclare_simple_queryable(
     res: &mut Arc<Resource>,
     send_declare: &mut SendDeclare,
 ) {
-    if !face_hat_mut!(face)
-        .remote_qabls
-        .values()
-        .any(|s| *s == *res)
-    {
-        if let Some(ctx) = get_mut_unchecked(res).session_ctxs.get_mut(&face.id) {
-            get_mut_unchecked(ctx).qabl = None;
-        }
+    let remote_qabl_info = get_remote_qabl_info(&face_hat_mut!(face).remote_qabls, res);
 
+    if update_queryable_info(res, face.id, &remote_qabl_info) {
         let mut simple_qabls = simple_qabls(res);
         if simple_qabls.is_empty() {
             propagate_forget_simple_queryable(tables, res, send_declare);
@@ -343,7 +349,7 @@ fn forget_simple_queryable(
     id: QueryableId,
     send_declare: &mut SendDeclare,
 ) -> Option<Arc<Resource>> {
-    if let Some(mut res) = face_hat_mut!(face).remote_qabls.remove(&id) {
+    if let Some((mut res, _)) = face_hat_mut!(face).remote_qabls.remove(&id) {
         undeclare_simple_queryable(tables, face, &mut res, send_declare);
         Some(res)
     } else {
@@ -363,7 +369,7 @@ pub(super) fn queries_new_face(
             .cloned()
             .collect::<Vec<Arc<FaceState>>>()
         {
-            for qabl in face_hat!(src_face).remote_qabls.values() {
+            for (ref qabl, _) in face_hat!(src_face).remote_qabls.values() {
                 propagate_simple_queryable_to(
                     tables,
                     face,
@@ -420,7 +426,7 @@ pub(super) fn declare_qabl_interest(
                         && face_hat!(src_face)
                             .remote_qabls
                             .values()
-                            .any(|qabl| qabl.context.is_some() && qabl.matches(res))
+                            .any(|(ref qabl, _)| qabl.context.is_some() && qabl.matches(res))
                 }) {
                     let info = local_qabl_info(tables, res, face);
                     let id = make_qabl_id(res, face, mode, info);
@@ -452,7 +458,7 @@ pub(super) fn declare_qabl_interest(
                     .collect::<Vec<Arc<FaceState>>>()
                 {
                     if src_face.id != face.id {
-                        for qabl in face_hat!(src_face).remote_qabls.values() {
+                        for (ref qabl, _) in face_hat!(src_face).remote_qabls.values() {
                             if qabl.context.is_some() && qabl.matches(res) {
                                 let info = local_qabl_info(tables, qabl, face);
                                 let id = make_qabl_id(qabl, face, mode, info);
@@ -487,37 +493,33 @@ pub(super) fn declare_qabl_interest(
             for src_face in tables
                 .faces
                 .values()
+                .filter(|f| f.id != face.id)
                 .cloned()
                 .collect::<Vec<Arc<FaceState>>>()
             {
-                if src_face.id != face.id {
-                    for qabl in face_hat!(src_face).remote_qabls.values() {
-                        if qabl.context.is_some() {
-                            let info = local_qabl_info(tables, qabl, face);
-                            let id = make_qabl_id(qabl, face, mode, info);
-                            let key_expr = Resource::decl_key(
-                                qabl,
-                                face,
-                                super::push_declaration_profile(face),
-                            );
-                            send_declare(
-                                &face.primitives,
-                                RoutingContext::with_expr(
-                                    Declare {
-                                        interest_id,
-                                        ext_qos: ext::QoSType::DECLARE,
-                                        ext_tstamp: None,
-                                        ext_nodeid: ext::NodeIdType::DEFAULT,
-                                        body: DeclareBody::DeclareQueryable(DeclareQueryable {
-                                            id,
-                                            wire_expr: key_expr,
-                                            ext_info: info,
-                                        }),
-                                    },
-                                    qabl.expr().to_string(),
-                                ),
-                            );
-                        }
+                for (ref qabl, _) in face_hat!(src_face).remote_qabls.values() {
+                    if qabl.context.is_some() {
+                        let info = local_qabl_info(tables, qabl, face);
+                        let id = make_qabl_id(qabl, face, mode, info);
+                        let key_expr =
+                            Resource::decl_key(qabl, face, super::push_declaration_profile(face));
+                        send_declare(
+                            &face.primitives,
+                            RoutingContext::with_expr(
+                                Declare {
+                                    interest_id,
+                                    ext_qos: ext::QoSType::DECLARE,
+                                    ext_tstamp: None,
+                                    ext_nodeid: ext::NodeIdType::DEFAULT,
+                                    body: DeclareBody::DeclareQueryable(DeclareQueryable {
+                                        id,
+                                        wire_expr: key_expr,
+                                        ext_info: info,
+                                    }),
+                                },
+                                qabl.expr().to_string(),
+                            ),
+                        );
                     }
                 }
             }
@@ -555,7 +557,7 @@ impl HatQueriesTrait for HatCode {
         // Compute the list of known queryables (keys)
         let mut qabls = HashMap::new();
         for face in tables.faces.values() {
-            for qabl in face_hat!(face).remote_qabls.values() {
+            for (ref qabl, _) in face_hat!(face).remote_qabls.values() {
                 // Insert the key in the list of known queryables
                 let srcs = qabls.entry(qabl.clone()).or_insert_with(Sources::empty);
                 // Append src_face as a queryable source in the proper list
@@ -601,75 +603,27 @@ impl HatQueriesTrait for HatCode {
     fn compute_query_route(
         &self,
         tables: &Tables,
-        expr: &mut RoutingExpr,
+        expr: &RoutingExpr,
         source: NodeId,
         source_type: WhatAmI,
     ) -> Arc<QueryTargetQablSet> {
         let mut route = QueryTargetQablSet::new();
-        let key_expr = expr.full_expr();
-        if key_expr.ends_with('/') {
+        let Some(key_expr) = expr.key_expr() else {
             return EMPTY_ROUTE.clone();
-        }
+        };
         tracing::trace!(
             "compute_query_route({}, {:?}, {:?})",
             key_expr,
             source,
             source_type
         );
-        let key_expr = match OwnedKeyExpr::try_from(key_expr) {
-            Ok(ke) => ke,
-            Err(e) => {
-                tracing::warn!("Invalid KE reached the system: {}", e);
-                return EMPTY_ROUTE.clone();
-            }
-        };
 
-        if source_type == WhatAmI::Client {
-            // TODO: BestMatching: What if there is a local compete ?
-            for face in tables
-                .faces
-                .values()
-                .filter(|f| f.whatami == WhatAmI::Router)
-            {
-                if !face.local_interests.values().any(|interest| {
-                    interest.finalized
-                        && interest.options.queryables()
-                        && interest
-                            .res
-                            .as_ref()
-                            .map(|res| KeyExpr::keyexpr_include(res.expr(), expr.full_expr()))
-                            .unwrap_or(true)
-                }) || face_hat!(face)
-                    .remote_qabls
-                    .values()
-                    .any(|sub| KeyExpr::keyexpr_intersect(sub.expr(), expr.full_expr()))
-                {
-                    let key_expr = Resource::get_best_key(expr.prefix, expr.suffix, face.id);
-                    route.push(QueryTargetQabl {
-                        direction: (face.clone(), key_expr.to_owned(), NodeId::default()),
-                        info: None,
-                    });
-                }
-            }
-
-            for face in tables.faces.values().filter(|f| {
-                f.whatami == WhatAmI::Peer
-                    && !initial_interest(f).map(|i| i.finalized).unwrap_or(true)
-            }) {
-                let key_expr = Resource::get_best_key(expr.prefix, expr.suffix, face.id);
-                route.push(QueryTargetQabl {
-                    direction: (face.clone(), key_expr.to_owned(), NodeId::default()),
-                    info: None,
-                });
-            }
-        }
-
-        let res = Resource::get_resource(expr.prefix, expr.suffix);
-        let matches = res
+        let matches = expr
+            .resource()
             .as_ref()
             .and_then(|res| res.context.as_ref())
             .map(|ctx| Cow::from(&ctx.matches))
-            .unwrap_or_else(|| Cow::from(Resource::get_matches(tables, &key_expr)));
+            .unwrap_or_else(|| Cow::from(Resource::get_matches(tables, key_expr)));
 
         for mres in matches.iter() {
             let mres = mres.upgrade().unwrap();
@@ -682,6 +636,34 @@ impl HatQueriesTrait for HatCode {
                 }
             }
         }
+
+        if source_type == WhatAmI::Client {
+            // TODO: BestMatching: What if there is a local compete ?
+            for face in tables.faces.values() {
+                if face.whatami == WhatAmI::Router {
+                    let has_interest_finalized = expr
+                        .resource()
+                        .and_then(|res| res.session_ctxs.get(&face.id))
+                        .is_some_and(|ctx| ctx.queryable_interest_finalized);
+                    if !has_interest_finalized {
+                        let wire_expr = expr.get_best_key(face.id);
+                        route.push(QueryTargetQabl {
+                            direction: (face.clone(), wire_expr.to_owned(), NodeId::default()),
+                            info: None,
+                        });
+                    }
+                } else if face.whatami == WhatAmI::Peer
+                    && initial_interest(face).is_some_and(|i| !i.finalized)
+                {
+                    let wire_expr = expr.get_best_key(face.id);
+                    route.push(QueryTargetQabl {
+                        direction: (face.clone(), wire_expr.to_owned(), NodeId::default()),
+                        info: None,
+                    });
+                }
+            }
+        }
+
         route.sort_by_key(|qabl| qabl.info.map_or(u16::MAX, |i| i.distance));
         Arc::new(route)
     }
