@@ -517,7 +517,7 @@ impl HatBaseTrait for HatCode {
         }
 
         let mut qabls_matches = vec![];
-        for (_, mut res) in hat_face.remote_qabls.drain() {
+        for (_, (mut res, _)) in hat_face.remote_qabls.drain() {
             get_mut_unchecked(&mut res).session_ctxs.remove(&face.id);
             undeclare_simple_queryable(&mut wtables, &mut face_clone, &mut res, send_declare);
 
@@ -790,13 +790,13 @@ impl HatBaseTrait for HatCode {
     }
 
     #[inline]
-    fn ingress_filter(&self, tables: &Tables, face: &FaceState, expr: &mut RoutingExpr) -> bool {
+    fn ingress_filter(&self, tables: &Tables, face: &FaceState, expr: &RoutingExpr) -> bool {
         face.whatami != WhatAmI::Peer
             || hat!(tables).linkstatepeers_net.is_none()
             || tables.zid
                 == *hat!(tables).elect_router(
                     &tables.zid,
-                    expr.full_expr(),
+                    expr.key_expr().map_or("", |ke| ke),
                     hat!(tables).get_router_links(face.zid),
                 )
     }
@@ -807,7 +807,7 @@ impl HatBaseTrait for HatCode {
         tables: &Tables,
         src_face: &FaceState,
         out_face: &Arc<FaceState>,
-        expr: &mut RoutingExpr,
+        expr: &RoutingExpr,
     ) -> bool {
         if src_face.id != out_face.id
             && (out_face.mcast_group.is_none() || src_face.mcast_group.is_none())
@@ -817,7 +817,7 @@ impl HatBaseTrait for HatCode {
                 || tables.zid
                     == *hat!(tables).elect_router(
                         &tables.zid,
-                        expr.full_expr(),
+                        expr.key_expr().map_or("", |ke| ke),
                         hat!(tables).get_router_links(out_face.zid),
                     );
 
@@ -948,7 +948,7 @@ struct HatFace {
     local_subs: HashMap<Arc<Resource>, SubscriberId>,
     remote_subs: HashMap<SubscriberId, Arc<Resource>>,
     local_qabls: HashMap<Arc<Resource>, (QueryableId, QueryableInfoType)>,
-    remote_qabls: HashMap<QueryableId, Arc<Resource>>,
+    remote_qabls: HashMap<QueryableId, (Arc<Resource>, QueryableInfoType)>,
     local_tokens: HashMap<Arc<Resource>, TokenId>,
     remote_tokens: HashMap<TokenId, Arc<Resource>>,
 }
