@@ -20,10 +20,7 @@ use std::{
 use petgraph::graph::NodeIndex;
 use zenoh_protocol::{
     core::{
-        key_expr::{
-            include::{Includer, DEFAULT_INCLUDER},
-            OwnedKeyExpr,
-        },
+        key_expr::include::{Includer, DEFAULT_INCLUDER},
         WhatAmI, ZenohIdProto,
     },
     network::{
@@ -51,19 +48,16 @@ use crate::{
                 tables::{QueryTargetQabl, QueryTargetQablSet, RoutingExpr, Tables},
             },
             hat::{CurrentFutureTrait, HatQueriesTrait, SendDeclare, Sources},
-            router::disable_matches_query_routes,
+            router::{
+                disable_matches_query_routes, get_remote_qabl_info, merge_qabl_infos,
+                update_queryable_info,
+            },
             RoutingContext,
         },
     },
 };
 
 #[inline]
-fn merge_qabl_infos(mut this: QueryableInfoType, info: &QueryableInfoType) -> QueryableInfoType {
-    this.complete = this.complete || info.complete;
-    this.distance = std::cmp::min(this.distance, info.distance);
-    this
-}
-
 fn local_router_qabl_info(tables: &Tables, res: &Arc<Resource>) -> QueryableInfoType {
     let info = if hat!(tables).full_net(WhatAmI::Peer) {
         res.context.as_ref().and_then(|_| {
@@ -99,6 +93,7 @@ fn local_router_qabl_info(tables: &Tables, res: &Arc<Resource>) -> QueryableInfo
         .unwrap_or(QueryableInfoType::DEFAULT)
 }
 
+#[inline]
 fn local_peer_qabl_info(tables: &Tables, res: &Arc<Resource>) -> QueryableInfoType {
     let info = if res.context.is_some() {
         res_hat!(res)
@@ -132,6 +127,7 @@ fn local_peer_qabl_info(tables: &Tables, res: &Arc<Resource>) -> QueryableInfoTy
         .unwrap_or(QueryableInfoType::DEFAULT)
 }
 
+#[inline]
 fn local_qabl_info(
     tables: &Tables,
     res: &Arc<Resource>,
@@ -237,6 +233,39 @@ fn send_sourced_queryable_to_net_children(
     }
 }
 
+#[inline]
+fn send_declare_queryable(
+    tables: &Tables,
+    dst_face: &mut Arc<FaceState>,
+    res: &Arc<Resource>,
+    id: u32,
+    info: QueryableInfoType,
+    send_declare: &mut SendDeclare,
+) {
+    face_hat_mut!(dst_face)
+        .local_qabls
+        .insert(res.clone(), (id, info));
+    let push_declaration = push_declaration_profile(tables, dst_face);
+    let key_expr = Resource::decl_key(res, dst_face, push_declaration);
+    send_declare(
+        &dst_face.primitives,
+        RoutingContext::with_expr(
+            Declare {
+                interest_id: None,
+                ext_qos: ext::QoSType::DECLARE,
+                ext_tstamp: None,
+                ext_nodeid: ext::NodeIdType::DEFAULT,
+                body: DeclareBody::DeclareQueryable(DeclareQueryable {
+                    id,
+                    wire_expr: key_expr,
+                    ext_info: info,
+                }),
+            },
+            res.expr().to_string(),
+        ),
+    );
+}
+
 fn propagate_simple_queryable(
     tables: &mut Tables,
     res: &Arc<Resource>,
@@ -246,17 +275,10 @@ fn propagate_simple_queryable(
     let full_peers_net = hat!(tables).full_net(WhatAmI::Peer);
     let faces = tables.faces.values().cloned();
     for mut dst_face in faces {
-        let info = local_qabl_info(tables, res, &dst_face);
-        let current = face_hat!(dst_face).local_qabls.get(res);
         if src_face
             .as_ref()
             .map(|src_face| dst_face.id != src_face.id)
             .unwrap_or(true)
-            && (current.is_none() || current.unwrap().1 != info)
-            && face_hat!(dst_face)
-                .remote_interests
-                .values()
-                .any(|i| i.options.queryables() && i.matches(res))
             && if full_peers_net {
                 dst_face.whatami == WhatAmI::Client
             } else {
@@ -271,31 +293,26 @@ fn propagate_simple_queryable(
                         .unwrap_or(true)
             }
         {
-            let id = current
-                .map(|c| c.0)
-                .unwrap_or(face_hat!(dst_face).next_id.fetch_add(1, Ordering::SeqCst));
-            face_hat_mut!(&mut dst_face)
-                .local_qabls
-                .insert(res.clone(), (id, info));
-            let push_declaration = push_declaration_profile(tables, &dst_face);
-            let key_expr = Resource::decl_key(res, &mut dst_face, push_declaration);
-            send_declare(
-                &dst_face.primitives,
-                RoutingContext::with_expr(
-                    Declare {
-                        interest_id: None,
-                        ext_qos: ext::QoSType::DECLARE,
-                        ext_tstamp: None,
-                        ext_nodeid: ext::NodeIdType::DEFAULT,
-                        body: DeclareBody::DeclareQueryable(DeclareQueryable {
-                            id,
-                            wire_expr: key_expr,
-                            ext_info: info,
-                        }),
-                    },
-                    res.expr().to_string(),
-                ),
-            );
+            if let Some(&(current_id, current_info)) = face_hat!(dst_face).local_qabls.get(res) {
+                let info = local_qabl_info(tables, res, &dst_face);
+                if current_info != info
+                    && face_hat!(dst_face)
+                        .remote_interests
+                        .values()
+                        .any(|i| i.options.queryables() && i.matches(res))
+                {
+                    let id = current_id;
+                    send_declare_queryable(tables, &mut dst_face, res, id, info, send_declare);
+                }
+            } else if face_hat!(dst_face)
+                .remote_interests
+                .values()
+                .any(|i| i.options.queryables() && i.matches(res))
+            {
+                let info = local_qabl_info(tables, res, &dst_face);
+                let id = face_hat!(dst_face).next_id.fetch_add(1, Ordering::SeqCst);
+                send_declare_queryable(tables, &mut dst_face, res, id, info, send_declare);
+            }
         }
     }
 }
@@ -448,7 +465,9 @@ fn register_simple_queryable(
         )
         .qabl = Some(*qabl_info);
     }
-    face_hat_mut!(face).remote_qabls.insert(id, res.clone());
+    face_hat_mut!(face)
+        .remote_qabls
+        .insert(id, (res.clone(), *qabl_info));
 }
 
 fn declare_simple_queryable(
@@ -789,15 +808,9 @@ pub(super) fn undeclare_simple_queryable(
     res: &mut Arc<Resource>,
     send_declare: &mut SendDeclare,
 ) {
-    if !face_hat_mut!(face)
-        .remote_qabls
-        .values()
-        .any(|s| *s == *res)
-    {
-        if let Some(ctx) = get_mut_unchecked(res).session_ctxs.get_mut(&face.id) {
-            get_mut_unchecked(ctx).qabl = None;
-        }
+    let remote_qabl_info = get_remote_qabl_info(&face_hat_mut!(face).remote_qabls, res);
 
+    if update_queryable_info(res, face.id, &remote_qabl_info) {
         let mut simple_qabls = simple_qabls(res);
         let router_qabls = remote_router_qabls(tables, res);
         let linkstatepeer_qabls = remote_linkstatepeer_qabls(tables, res);
@@ -874,7 +887,7 @@ fn forget_simple_queryable(
     id: QueryableId,
     send_declare: &mut SendDeclare,
 ) -> Option<Arc<Resource>> {
-    if let Some(mut res) = face_hat_mut!(face).remote_qabls.remove(&id) {
+    if let Some((mut res, _)) = face_hat_mut!(face).remote_qabls.remove(&id) {
         undeclare_simple_queryable(tables, face, &mut res, send_declare);
         Some(res)
     } else {
@@ -999,7 +1012,7 @@ pub(super) fn queries_linkstate_change(
                 if src_face.id != dst_face.id
                     && HatTables::failover_brokering_to(links, &dst_face.zid)
                 {
-                    for res in face_hat!(src_face).remote_qabls.values() {
+                    for (ref res, _) in face_hat!(src_face).remote_qabls.values() {
                         if !face_hat!(dst_face).local_qabls.contains_key(res) {
                             let id = face_hat!(dst_face).next_id.fetch_add(1, Ordering::SeqCst);
                             let info = local_qabl_info(tables, res, &dst_face);
@@ -1082,7 +1095,7 @@ pub(super) fn queries_tree_change(
 #[inline]
 fn insert_target_for_qabls(
     route: &mut QueryTargetQablSet,
-    expr: &mut RoutingExpr,
+    expr: &RoutingExpr,
     tables: &Tables,
     net: &Network,
     source: NodeId,
@@ -1098,10 +1111,9 @@ fn insert_target_for_qabls(
                         if net.graph.contains_node(direction) {
                             if let Some(face) = tables.get_face(&net.graph[direction].zid) {
                                 if net.distances.len() > qabl_idx.index() {
-                                    let key_expr =
-                                        Resource::get_best_key(expr.prefix, expr.suffix, face.id);
+                                    let wire_expr = expr.get_best_key(face.id);
                                     route.push(QueryTargetQabl {
-                                        direction: (face.clone(), key_expr.to_owned(), source),
+                                        direction: (face.clone(), wire_expr.to_owned(), source),
                                         info: Some(QueryableInfoType {
                                             complete: complete && qabl_info.complete,
                                             distance: net.distances[qabl_idx.index()] as u16,
@@ -1426,37 +1438,29 @@ impl HatQueriesTrait for HatCode {
     fn compute_query_route(
         &self,
         tables: &Tables,
-        expr: &mut RoutingExpr,
+        expr: &RoutingExpr,
         source: NodeId,
         source_type: WhatAmI,
     ) -> Arc<QueryTargetQablSet> {
         let mut route = QueryTargetQablSet::new();
-        let key_expr = expr.full_expr();
-        if key_expr.ends_with('/') {
+        let Some(key_expr) = expr.key_expr() else {
             return EMPTY_ROUTE.clone();
-        }
+        };
         tracing::trace!(
             "compute_query_route({}, {:?}, {:?})",
             key_expr,
             source,
             source_type
         );
-        let key_expr = match OwnedKeyExpr::try_from(key_expr) {
-            Ok(ke) => ke,
-            Err(e) => {
-                tracing::warn!("Invalid KE reached the system: {}", e);
-                return EMPTY_ROUTE.clone();
-            }
-        };
-        let res = Resource::get_resource(expr.prefix, expr.suffix);
-        let matches = res
+        let matches = expr
+            .resource()
             .as_ref()
             .and_then(|res| res.context.as_ref())
             .map(|ctx| Cow::from(&ctx.matches))
-            .unwrap_or_else(|| Cow::from(Resource::get_matches(tables, &key_expr)));
+            .unwrap_or_else(|| Cow::from(Resource::get_matches(tables, key_expr)));
 
         let master = !hat!(tables).full_net(WhatAmI::Peer)
-            || *hat!(tables).elect_router(&tables.zid, &key_expr, hat!(tables).shared_nodes.iter())
+            || *hat!(tables).elect_router(&tables.zid, key_expr, hat!(tables).shared_nodes.iter())
                 == tables.zid;
 
         for mres in matches.iter() {
