@@ -36,7 +36,7 @@ use super::{
 use crate::net::{
     protocol::{linkstate::LinkEdgeWeight, network::Network},
     routing::{
-        dispatcher::{face::FaceState, interests::RemoteInterest, tables::Tables},
+        dispatcher::{face::FaceState, tables::Tables},
         hat::{CurrentFutureTrait, HatTokenTrait, SendDeclare},
         router::{NodeId, Resource, SessionContext},
         RoutingContext,
@@ -105,48 +105,31 @@ fn propagate_simple_token_to(
                     || dst_face.whatami != WhatAmI::Peer
                     || hat!(tables).failover_brokering(src_face.zid, dst_face.zid))
         }
-    {
-        let matching_interests = face_hat!(dst_face)
+        && face_hat!(dst_face)
             .remote_interests
             .values()
-            .filter(|i| i.options.tokens() && i.matches(res))
-            .cloned()
-            .collect::<Vec<_>>();
-
-        for RemoteInterest {
-            res: int_res,
-            options,
-            ..
-        } in matching_interests
-        {
-            let res = if options.aggregate() {
-                int_res.as_ref().unwrap_or(res)
-            } else {
-                res
-            };
-            if !face_hat!(dst_face).local_tokens.contains_key(res) {
-                let id = face_hat!(dst_face).next_id.fetch_add(1, Ordering::SeqCst);
-                face_hat_mut!(dst_face).local_tokens.insert(res.clone(), id);
-                let key_expr =
-                    Resource::decl_key(res, dst_face, push_declaration_profile(tables, dst_face));
-                send_declare(
-                    &dst_face.primitives,
-                    RoutingContext::with_expr(
-                        Declare {
-                            interest_id: None,
-                            ext_qos: ext::QoSType::DECLARE,
-                            ext_tstamp: None,
-                            ext_nodeid: ext::NodeIdType::DEFAULT,
-                            body: DeclareBody::DeclareToken(DeclareToken {
-                                id,
-                                wire_expr: key_expr,
-                            }),
-                        },
-                        res.expr().to_string(),
-                    ),
-                );
-            }
-        }
+            .any(|i| i.options.tokens() && i.matches(res))
+    {
+        let id = face_hat!(dst_face).next_id.fetch_add(1, Ordering::SeqCst);
+        face_hat_mut!(dst_face).local_tokens.insert(res.clone(), id);
+        let key_expr =
+            Resource::decl_key(res, dst_face, push_declaration_profile(tables, dst_face));
+        send_declare(
+            &dst_face.primitives,
+            RoutingContext::with_expr(
+                Declare {
+                    interest_id: None,
+                    ext_qos: ext::QoSType::DECLARE,
+                    ext_tstamp: None,
+                    ext_nodeid: ext::NodeIdType::DEFAULT,
+                    body: DeclareBody::DeclareToken(DeclareToken {
+                        id,
+                        wire_expr: key_expr,
+                    }),
+                },
+                res.expr().to_string(),
+            ),
+        );
     }
 }
 
@@ -433,7 +416,7 @@ fn propagate_forget_simple_token(
         }) && face_hat!(face)
             .remote_interests
             .values()
-            .any(|i| i.options.tokens() && i.matches(res) && !i.options.aggregate())
+            .any(|i| i.options.tokens() && i.matches(res))
         {
             // Token has never been declared on this face.
             // Send an Undeclare with a one shot generated id and a WireExpr ext.
@@ -490,7 +473,7 @@ fn propagate_forget_simple_token(
                 } else if face_hat!(face)
                     .remote_interests
                     .values()
-                    .any(|i| i.options.tokens() && i.matches(&res) && !i.options.aggregate())
+                    .any(|i| i.options.tokens() && i.matches(&res))
                     && src_face.map_or(true, |src_face| {
                         src_face.whatami != WhatAmI::Peer
                             || face.whatami != WhatAmI::Peer
@@ -995,7 +978,6 @@ pub(crate) fn declare_token_interest(
     id: InterestId,
     res: Option<&mut Arc<Resource>>,
     mode: InterestMode,
-    aggregate: bool,
     send_declare: &mut SendDeclare,
 ) {
     if mode.current()
@@ -1004,17 +986,29 @@ pub(crate) fn declare_token_interest(
     {
         let interest_id = Some(id);
         if let Some(res) = res.as_ref() {
-            if aggregate {
-                if hat!(tables).router_tokens.iter().any(|token| {
-                    token.context.is_some()
-                        && token.matches(res)
-                        && (remote_simple_tokens(tables, token, face)
-                            || remote_linkstatepeer_tokens(tables, token)
-                            || remote_router_tokens(tables, token))
-                }) {
-                    let id = make_token_id(res, face, mode);
+            for token in &hat!(tables).router_tokens {
+                if token.context.is_some()
+                    && token.matches(res)
+                    && (res_hat!(token)
+                        .router_tokens
+                        .iter()
+                        .any(|r| *r != tables.zid)
+                        || res_hat!(token)
+                            .linkstatepeer_tokens
+                            .iter()
+                            .any(|r| *r != tables.zid)
+                        || token.session_ctxs.values().any(|s| {
+                            s.face.id != face.id
+                                && s.token
+                                && (s.face.whatami == WhatAmI::Client
+                                    || face.whatami == WhatAmI::Client
+                                    || (s.face.whatami == WhatAmI::Peer
+                                        && hat!(tables).failover_brokering(s.face.zid, face.zid)))
+                        }))
+                {
+                    let id = make_token_id(token, face, mode);
                     let wire_expr =
-                        Resource::decl_key(res, face, push_declaration_profile(tables, face));
+                        Resource::decl_key(token, face, push_declaration_profile(tables, face));
                     send_declare(
                         &face.primitives,
                         RoutingContext::with_expr(
@@ -1025,50 +1019,10 @@ pub(crate) fn declare_token_interest(
                                 ext_nodeid: ext::NodeIdType::DEFAULT,
                                 body: DeclareBody::DeclareToken(DeclareToken { id, wire_expr }),
                             },
-                            res.expr().to_string(),
+                            token.expr().to_string(),
                         ),
                     );
                 }
-            } else {
-                for token in &hat!(tables).router_tokens {
-                    if token.context.is_some()
-                        && token.matches(res)
-                        && (res_hat!(token)
-                            .router_tokens
-                            .iter()
-                            .any(|r| *r != tables.zid)
-                            || res_hat!(token)
-                                .linkstatepeer_tokens
-                                .iter()
-                                .any(|r| *r != tables.zid)
-                            || token.session_ctxs.values().any(|s| {
-                                s.face.id != face.id
-                                    && s.token
-                                    && (s.face.whatami == WhatAmI::Client
-                                        || face.whatami == WhatAmI::Client
-                                        || (s.face.whatami == WhatAmI::Peer
-                                            && hat!(tables)
-                                                .failover_brokering(s.face.zid, face.zid)))
-                            }))
-                    {
-                        let id = make_token_id(token, face, mode);
-                        let wire_expr =
-                            Resource::decl_key(token, face, push_declaration_profile(tables, face));
-                        send_declare(
-                            &face.primitives,
-                            RoutingContext::with_expr(
-                                Declare {
-                                    interest_id,
-                                    ext_qos: ext::QoSType::DECLARE,
-                                    ext_tstamp: None,
-                                    ext_nodeid: ext::NodeIdType::DEFAULT,
-                                    body: DeclareBody::DeclareToken(DeclareToken { id, wire_expr }),
-                                },
-                                token.expr().to_string(),
-                            ),
-                        );
-                    }
-                }
             }
         } else {
             for token in &hat!(tables).router_tokens {
