@@ -23,7 +23,7 @@ use std::{
 };
 
 use itertools::Itertools;
-use tokio_util::sync::CancellationToken;
+use tokio_util::{sync::CancellationToken, task::TaskTracker};
 use zenoh::{config::WhatAmI, qos::CongestionControl, Config, Result, Session};
 use zenoh_config::{ModeDependentValue, WhatAmIMatcher};
 use zenoh_core::ztimeout;
@@ -433,7 +433,7 @@ impl Recipe {
         loop {
             tokio::select! {
                 _ = tokio::time::sleep(TIMEOUT) => {
-                    println!("Recipe {} Timeout.", self);
+                    println!("Recipe {self} Timeout.");
 
                     // Termination
                     remaining_checkpoints.swap(0, Ordering::Relaxed);
@@ -453,7 +453,7 @@ impl Recipe {
             }
         }
 
-        println!("Recipe {} OK.", self);
+        println!("Recipe {self} OK.");
         Ok(())
     }
 }
@@ -510,7 +510,7 @@ async fn gossip() -> Result<()> {
     // - node1: Peer, node2: Peer, node3: Router
     for mode in [WhatAmI::Peer, WhatAmI::Router] {
         let node3 = Node {
-            name: format!("Router {}", mode),
+            name: format!("Router {mode}"),
             mode: WhatAmI::Peer,
             listen: vec![locator.clone()],
             con_task: ConcurrentTask::from([SequentialTask::from([Task::Sleep(
@@ -581,9 +581,6 @@ async fn static_failover_brokering() -> Result<()> {
     Result::Ok(())
 }
 
-#[cfg(feature = "unstable")]
-use tokio_util::task::TaskTracker;
-#[cfg(feature = "unstable")]
 const MSG_SIZE: [usize; 2] = [1_024, 131_072];
 // Maximal recipes to run at once
 const PARALLEL_RECIPES: usize = 4;
@@ -594,7 +591,6 @@ const PARALLEL_RECIPES: usize = 4;
 // 3. Spawning order (delay_in_secs for node1, node2, and node3) = 6 (cases)
 //
 // Total cases = 2 x 4 x 6 = 48
-#[cfg(feature = "unstable")]
 #[tokio::test(flavor = "multi_thread", worker_threads = 9)]
 async fn three_node_combination() -> Result<()> {
     zenoh::init_log_from_env_or("error");
@@ -771,7 +767,6 @@ async fn three_node_combination() -> Result<()> {
 // 2. Mode: {Client, Peer} x {Client, Peer} x {IsFirstListen} = 2 x 2 x 2 = 8 (modes)
 //
 // Total cases = 2 x 8 = 16
-#[cfg(feature = "unstable")]
 #[tokio::test(flavor = "multi_thread", worker_threads = 8)]
 async fn two_node_combination() -> Result<()> {
     zenoh::init_log_from_env_or("error");
@@ -923,6 +918,7 @@ async fn two_node_combination() -> Result<()> {
     Result::Ok(())
 }
 
+#[ignore = "This is a flaky test"]
 #[tokio::test(flavor = "multi_thread", worker_threads = 9)]
 async fn three_node_combination_multicast() -> Result<()> {
     zenoh::try_init_log_from_env();
@@ -1042,7 +1038,6 @@ async fn three_node_combination_multicast() -> Result<()> {
     Ok(())
 }
 
-#[cfg(feature = "unstable")]
 #[tokio::test(flavor = "multi_thread", worker_threads = 9)]
 async fn peer_linkstate() -> Result<()> {
     zenoh_util::try_init_log_from_env();
@@ -1223,7 +1218,6 @@ async fn peer_linkstate() -> Result<()> {
     Ok(())
 }
 
-#[cfg(feature = "unstable")]
 #[tokio::test(flavor = "multi_thread", worker_threads = 9)]
 async fn router_linkstate() -> Result<()> {
     zenoh_util::try_init_log_from_env();
